extern "C" {
    #include <sys/stat.h>

    #include <errno.h>
	#include <fcntl.h>
	#include <stdio.h>
	#include <string.h>
    #include <unistd.h>
};

#include <gtkmm/buttonbox.h>
#include <gtkmm/entry.h>
#include <gtkmm/menu.h>
#include <gtkmm/menushell.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/widget.h>
#include <sigc++/slot.h>

#include "gui.hh"
#include "globals.hh"
#include "serial.hh"
#include "strings.hh"

char Gui::errmess[Gui::ERRBUFSIZE];

SetupPage::SetupPage(OptionTable& options, Gtk::Tooltips& tips) :
  portBox(Gtk::FILE_CHOOSER_ACTION_OPEN)
{
	int row;
	
	set_border_width(5);
	set_spacing(1);

	// optionsOuterBox is the HBox containing all of he controls
	// located above the port box
	pack_start(optionsOuterBox, false, false);
	optionsOuterBox.set_border_width(5);
	optionsOuterBox.set_spacing(10);

	// optionsFrame is the frame around the boolean options selected
	// by checkboxes
	optionsOuterBox.pack_start(optionsFrame, false, false);

	// optionsLeftBox is the VBox containing the two rows of checkboxes
	optionsFrame.add(optionsLeftBox);
	optionsLeftBox.set_border_width(5);
	optionsLeftBox.set_spacing(10);

	// optionsInnerBox[] is the array of rows of checkboxes
	for (row = 0; row < 2; row++) {
		optionsLeftBox.add(optionsInnerBox[row]);
		optionsInnerBox[row].set_border_width(5);
		optionsInnerBox[row].set_spacing(10);
	}

	// optionsRightBox is the VBox containing the min-eeprom selection
	// and the comm font selection
	optionsOuterBox.pack_end(optionsRightBox, false, false);
	
	pack_end(portFrame, true, true);
	portFrame.set_shadow_type(Gtk::SHADOW_ETCHED_IN);
							   
	portFrame.add(portBox);
	portBox.set_action(Gtk::FILE_CHOOSER_ACTION_OPEN);
	portBox.set_select_multiple(false);
	// I'll be putting my own filter back later
	Gtk::FileFilter ttysonly;
	ttysonly.add_pattern("ttyS*");
	portBox.add_filter(ttysonly);
	
	// OK, now for the stuff we pull out of the options table
	row = 0;
	for (OptionTable::iterator i = options.begin(); i->longName() != NULL; ++i) {
	
		switch (i->argInfo()) {
		  case POPT_ARG_NONE:
		  case POPT_ARG_LONG: {
			  Gtk::CheckButton* newButton = new Gtk::CheckButton(i->longName(), 0.0);
			  i->setWidget(newButton);
			  optionsInnerBox[row++ % 2].add(*newButton);
			  newButton->set_active((i->getBool()) != false);
			  //			  newButton->toggled.connect(SigC::slot(this,
			  //													&SetupPage::changed));
			  newButton->signal_toggled().connect(sigc::mem_fun(this, &SetupPage::checkbox_toggled));
			  tips.set_tip(*newButton, i->descrip());
		  }
		  break;

		  case POPT_ARG_STRING:
			// Of our various string options, only a few are actually
			// handled by the general preferences window.  The others
			// are ignored here.
			
			switch (i->shortName()) {
			  case 'f': // file to download -- ignore, since that
						// isn't managed by the Preferences dialog
				break;
				
			  case 'p': // ports
				// >>> portBox.set_label(i->descrip);
				portFrame.set_label(i->descrip());
				
				// note that the checkFile signal has to be connected before the
				// path is set, so filenames get filtered properly.
				// The directory and filename are set separately so
				// the name list will get created and filtered when
				// set_directory is called, and then set_filename can
				// select the right file from the list.
				// >>> portBox.checkFile.connect(SigC::slot(&serial,
				//									 &Serial::checkDev));
				portBox.signal_selection_changed().connect(sigc::mem_fun(this,
														&SetupPage::port_changed));
				//portBox.unSelectedFile.connect(SigC::slot(this,
				//										  &SetupPage::changed));
				// >>>>				portBox.signal_selection_changed().connect(sigc::mem_fun(*this, &SetupPage::changed));
				
				// ummm, I'm supposed to be able to specify a default
				// when I define the option.  I'm getting a NULL
				// anyway, so I'll have to set the default by hand here
				if ((i->getCStr() != NULL) && (strlen(i->getCStr()) != 0))
					// >>> portBox.setPath(ustring(*(char **)i->arg));
					portBox.set_filename(i->getCStr());
				else
					portBox.set_current_folder("/dev/");
				i->setWidget(&portBox);
				
				//tips.set_tip(*(portSelect.gtk_entry()), i->descrip);

				//				if (*(char **)(i->arg) != 0)
				//					((Gtk::Entry *)(portSelect.gtk_entry()))->set_text(ustring(*(char **)(i->arg)));
				break;

			  default: // any other ustring options.  The only ones
				       // right now are min-eeprom and comm font
				Gtk::Frame *newFrame = new Gtk::Frame(i->longName());
				optionsRightBox.pack_start(*newFrame, false, false);
				Gtk::VButtonBox *newBox = new Gtk::VButtonBox;
				newBox->set_border_width(5);
				newBox->set_spacing(10);
				newFrame->add(*newBox);

				// the comm font selection and the min-eeprom entry
				// take some customization
				switch (i->shortName()) {
				  case 'c': // comm window font
					// >>> commFontPicker.set_mode(GNOME_FONT_PICKER_MODE_FONT_INFO);
					commFontPicker.set_title(str.CNSTFONTLBL.c_str());
					if (i->getCStr() != NULL) 
						commFontPicker.set_font_name(ustring(i->getCStr()));
					commFontPicker.set_use_font(true);
					newBox->pack_start(commFontPicker, false, false);
					
					// >>> commFontPicker.signal_font_set().connect(sigc::mem_fun(*this, &SetupPage::newFont));
					break;

				  case 'm': // min-eeprom
					Gtk::Entry *newEntry = new Gtk::Entry;
					if (i->getCStr() != NULL)
						newEntry->set_text(i->getCStr());
					else
						newEntry->set_text("0");
					newBox->pack_start(*newEntry, false, false);
					tips.set_tip(*newEntry, i->descrip());
					i->setWidget(newEntry);
					newEntry->signal_changed().connect(sigc::mem_fun(this, &SetupPage::changed));
				}
			}
		}
	}
}	

void SetupPage::changed()
{
	loader.checkPrereqs();
}

void SetupPage::checkbox_toggled()
{
	loader.checkPrereqs();
}

void SetupPage::port_changed()
{
	options['p'].set(getPortName());
	loader.checkPrereqs();
}

void SetupPage::newFont(gchar *fontName)
{
	int pxlsz;
	sscanf(fontName, "-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%d-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]", &pxlsz);
	commFontPicker.set_use_font(true);
	//if (gui != NULL)
	//	gui->preferences.changed();
}

// >>> this needs some more thought.  What I've  started here exports too mcuh information
// >>> regarding the GUI to the option  code; that's as bad as previously when I was putting
// >>> too much option info in the GUI
void SetupPage::apply(OptionTable* options) {
	for (OptionTable::iterator i = options->begin(); i->longName() != 0; ++i) {
		switch (i->argInfo()) {
		  case POPT_ARG_LONG:
		  case POPT_ARG_NONE:
			i->set(((Gtk::CheckButton *)(i->widget()))->get_active());
			break;

		  case POPT_ARG_STRING:
			// For the moment, we only have three string options:
			// min-eeprom and ports.  It's too bad there isn't a
			// ``file name''  option in popt; so, things are more
			// customized here than I'd like.
			switch (i->shortName()) {
				
			  case 'c': // font for communication window
				if (getCommFont() != "") {
					gui->setCommFont(getCommFont());
					i->set(getCommFont());
				}
				break;
				
			  case 'f': // file to download
			  case 'p': // ports
				i->set(((Gtk::FileChooserWidget *)(i->widget()))->get_filename());
				break;

			  default:  // just min-eeprom, for now
				i->set(((Gtk::Entry *)(i->widget()))->get_text());
				break;
			}
		}
	}
	options->write();
	// I don't even know what these two lines were for!
	// >>>Gdk::Rectangle::Rectangle wholewindow(0, 0, -1, -1);
	// >>> gui->mainWin.draw(wholewindow);
	//gui->preferences.draw(wholewindow);
	loader.checkPrereqs();
}

DownloadPage::DownloadPage(OptionTable& options, Gtk::Tooltips& tips) :
  fileBox(),
  downloadButton(str.DWNLDLBL),
  cancelButton(str.CNCLBL)
{
	set_border_width(5);
	set_spacing(10);
	
	// download file selector
	pack_start(fileFrame, true, true);
	fileFrame.add(fileBox);
	Gtk::FileFilter s19only;
	s19only.add_pattern("*.s19");
	fileBox.add_filter(s19only);
	
	// control buttons
	pack_end(downloadButtonBox, false, false);
	downloadButtonBox.set_border_width(5);
	downloadButtonBox.set_spacing(10);
	
	downloadButtonBox.pack_end(downloadButton, false, false);
	downloadButton.signal_clicked().connect(sigc::mem_fun(this, &DownloadPage::downloadPressed));
	
	downloadButtonBox.pack_end(cancelButton, false, false);
	cancelButton.signal_clicked().connect(sigc::mem_fun(this, &DownloadPage::cancelPressed));

	for (OptionTable::iterator i = options.begin(); i->longName() != 0; ++i)
		if (i->shortName() == 'f') {
			fileFrame.set_label(i->descrip());
			fileBox.signal_selection_changed().connect(sigc::mem_fun(this, &DownloadPage::fileSelected));

			// >>> fileBox.unSelectedFile.connect(SigC::slot(this,
			//													  &DownloadPage::fileUnSelected));
// >>> fileBox.checkFile.connect(SigC::slot(this, &DownloadPage::checks19file));
			// ummm, I'm supposed to be able to specify a default when
			// I define the option.  I'm getting a NULL anyway, so I
			// need to do the defaults by hand.
			if ((i->getCStr() != NULL) && (strlen(i->getCStr()) > 0))
				fileBox.set_filename(i->getCStr());
			else
				fileBox.set_filename(ustring(getcwd(NULL, 0)) + "/");
			i->setWidget(&fileBox);

			//>>> tips.set_tip(*(downloadSelect.gtk_entry()), i->descrip);
			//if (*(char **)(i->arg) != 0)
			//	((Gtk::Entry *)(fileBox.fname.gtk_entry()))->set_text(ustring(*(char **)(i->arg)));
		}
}

bool DownloadPage::checks19file(const ustring& dirName, ustring& fileName) {
	ustring fullName = dirName + fileName;

	// make sure it's a file with a name ending in s19, and that I can open it 
	
	gui->dbg(sprintf(gui->errmess, str.CHCKFILDBG.c_str(), fileName.c_str()));

	// check the name (if it doesn't match, we fail)
	char *suffix = rindex(fileName.c_str(), '.');
	//fprintf(stderr, "suffix %s\n", suffix);
	if (suffix == NULL)
		return(false);
	if ((strcmp(suffix, ".s19") != 0) &&
		(strcmp(suffix, ".S19") != 0))
		return(false);
	
	// stat the file (if we can't stat it, something is *seriously* wrong)
	struct stat filestat;

	int statreturn = stat(fullName.c_str(), &filestat);
	if (statreturn != 0)
		return(false);

	// is it a directory (if it is, we fail)
	if (S_ISDIR(filestat.st_mode) == true)
		return(false);

	// next, we'll see if we can open it for reading (this is what I
	// really want to know, after all, and it'll be a whole lot
	// simpler than a lot of fancy stat checks (if we can, it's a
	// useable s19 file)
	int fd = open(fullName.c_str(), O_RDONLY);
	if (fd > 0) {
		close(fd);
		return(true);
	}

	// it might actually be a good idea to show the file, but not
	// selectable or something, if we get here...
	return(false);
}

void DownloadPage::fileSelected(){
	// need to copy selected filename back to options table.
	options['f'].set(fileBox.get_filename());
	loader.checkPrereqs();
}

void DownloadPage::fileUnSelected(){
	loader.checkPrereqs();
}

void DownloadPage::cancelPressed() {
	if (gui != 0) {
		gui->setStatus(Gui::IDLE);
// >>>		gui->grab_remove(*this);
	}
}

void DownloadPage::downloadPressed() {
	gui->dbg(sprintf(gui->errmess, str.DNLDPRESDBG.c_str(),
					 gui->mainWin.controlPanel.setupPage.getPortName().c_str(),
					 fileBox.get_filename().c_str()));
	
// >>>gnome_config_set_ustring("/gdl/Options/file",
// >>>							fileBox.getPath().c_str());
// >>>	gnome_config_sync();
//	if (serial.openPort(gui->mainWin.controlPanel.setupPage.getPortName()))
	loader.downloadFile();
}

AppBar::AppBar() {
	pack_start(status, true, true);
	pack_end(progress, true, true);
}

void AppBar::setStatus(Glib::ustring stattext)
{
	status.set_text(stattext);
}

void AppBar::setProgress(double percentage)
{
	progress.set_fraction(percentage);
}
	
void MainWin::setStatus(ustring status) {

	gui->dbg(sprintf(gui->errmess, str.STATDBG.c_str(), status.c_str()));
	
	setProgress(0, 1);
	appBar.setStatus(status);

	// Ah, what a mess.  Having this loop is necessary to get the gui
	// to pay attention during a download.  But recent versions of GTK
	// seem to have introduced a priority inversion problem, in which
	// it's resulting in nested calls to setStatus; the inner call is
	// the second call made, but ends up being the first processed.
	// So the outer call "wins" in the end.  I need to move this loop
	// to the downloader itself.
	//	while (gui->events_pending() > 0)
	//		gui->iteration();
}

void MainWin::setProgress(int sofar, int total) {
	// >>>	appBar.get_progress()->set_percentage(((gfloat)sofar)/((gfloat)total));
	//	while (gui->events_pending() > 0)
	//				gui->iteration();
	
	appBar.setProgress(((double)sofar)/((double)total));

	// Same comment as loop in setStatus()
	//	while (gui->events_pending() > 0)
	//			gui->iteration();
}

ControlPanel::ControlPanel(OptionTable& options, Gtk::Tooltips& tips) :
  setupPage(options, tips),
  commPage(options),
  downloadPage(options, tips),
  currentPage(0)
 {
	 insert_page(setupPage, str.SETUPTABLBL, 0);
	 insert_page(downloadPage, str.DOWNTABLBL, 1);
	 insert_page(commPage, str.COMMTABLBL, 2);
	 signal_switch_page().connect(sigc::mem_fun(this, &ControlPanel::pageChanged));
}

void ControlPanel::pageChanged(GtkNotebookPage* page, guint pageNo)
{
	if (get_tab_label(*get_nth_page(pageNo))->is_sensitive()) {
		switch (pageNo) {
		  case 0: // setup page
			break;
			
		  case 1: // download page
			setupPage.apply(&options);
			commPage.setIO(false);
			loader.checkPrereqs();
			break;
			
		  case 2: // comm page
			//			setupPage.apply(&options);
			commPage.textArea.grab_focus();
			commPage.setIO(true);
			gui->setStatus(Gui::COMM);
			break;
		}
		currentPage = pageNo;
	} else
		set_current_page(currentPage);
}

void MainWin::DestroyHandler() {
	Gtk::Main::instance()->quit();
	exit(0);
}

void MainWin::ExitPressed() {
	options.write();
	exit(0);
}

void MainWin::ShowAbout()
{
	char const* nameArray[] = {
	  str.MYNAME.c_str(),
	  str.CAVEAT.c_str(),
	  NULL
	};
	
	// >>>	Gtk::SArray_Helpers::SArray author(nameArray);
	// >>> Gnome::About aboutThis(PACKAGE, VERSION, str.LICENSE, author);
	// >>> 	aboutThis.run();
}

void MainWin::ShowManual() {
	// >>> static GnomeHelpMenuEntry manualRef = {PACKAGE, "Manual.html"};
	// >>> gnome_help_display(NULL, &manualRef);
}
						 
MainWin::MainWin(OptionTable& options, Gtk::Tooltips& tips) :
  controlPanel(options, tips),
  // >>> Gnome::App(PACKAGE, PACKAGE),
  Gtk::Window(),
  appBar()
{
	// vbox holding everything
	add(vbox);
	// It seems like there ought to be a way to use item factories to
	// set up the menus, but I haven't found it...

	// File menu
	Gtk::Menu* menuFile = manage(new Gtk::Menu());
	Gtk::Menu_Helpers::MenuList& listFile = menuFile->items();

	listFile.push_back(Gtk::Menu_Helpers::MenuElem(str.EXITMENU,
												   //>>>  Gtk::Menu_Helpers::CTL | 'X',
											  sigc::ptr_fun(&MainWin::ExitPressed)));

	menubar.items().push_back(Gtk::Menu_Helpers::MenuElem(str.FILEMENU,
													 //>>Menu_Helpers::CTL | 'f',
													 *menuFile));

	// Now the Help menu
	//	>>>Gtk::Menu* menuHelp = manage(new Menu());
	//	>>>Gtk::Menu_Helpers::MenuList& listHelp = menuHelp->items();

	// >>> listHelp.push_back(Menu_Helpers::MenuElem(str.MANUALMENU,
	//								  Menu_Helpers::CTL | 'M',
	//									  SigC::slot(&MainWin::ShowManual)));

	// >>> listHelp.push_back(Menu_Helpers::MenuElem(str.ABOUTMENU,
	//								  Menu_Helpers::CTL | 'A',
	//									  SigC::slot(&MainWin::ShowAbout)));
	
	// >>>	menubar.items().push_back(Menu_Helpers::MenuElem(str.HELPMENU,
	//											 Menu_Helpers::CTL | 'H',
	//											 *menuHelp));
// >>>	menubar.items().back()->right_justify();
	
// >>>	set_policy(true, true, true);
// >>>	set_menus(menubar);
	vbox.pack_start(menubar, false, false);
	vbox.pack_start(controlPanel, true, true);
// >>>	set_contents(controlPanel);
// >>>	set_statusbar(appBar);
	vbox.pack_end(appBar, false, false);
	
// >>>	signal_destroy().connect(sigc::mem_fun(*this, &MainWin::DestroyHandler));
}

CommPage::CommPage(OptionTable& options)
  //  hadj(0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
  //  vadj(0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
  //  textScroll(vadj)
  // >>>,  textArea(hadj, vadj)

{
	fontTag = Gtk::TextBuffer::Tag::create();
	tagTable = textArea.get_buffer()->get_tag_table();
	tagTable->add(fontTag);
	
	serialTag = 0;
	//	add(windowFrame);
	//	windowFrame.add(textBox);
	add(textBox);
	textBox.add(textArea);
	//	textBox.signal_focus_in_event().connect(sigc::mem_fun(*this, &CommPage::place_cursor));

	// get the textarea style so we can manipulate its font.
	textAreaStyle = textArea.get_style()->copy();
	textArea.set_style(textAreaStyle);
	
	// >>>	textArea.key_press_event.connect(SigC::slot(this,&CommPage::sendCharOut));
	textArea.signal_key_press_event().connect(sigc::mem_fun(this, &CommPage::sendCharOut), false);
	// eventually this will change to be a 24x80 character window.
	textArea.set_size_request(640, 480);
	//textBox.pack_start(textArea, true, false);
	//textBox.pack_end(textScroll, false, false);
}

int CommPage::sendCharOut(GdkEventKey *ev) {
	// Hmmm....  latest gtkmm docs say that the only way to do text
	// input is with input contexts, which look like a mess to me.
	// This approach is deprecated, but for the moment I'll keep on
	// doing it...
	gui->dbg(sprintf(gui->errmess, str.SNDCHRDBG.c_str(), ev->string[0]));

	// catch is that there is a keypress event for every keypress, but
	// not every keypress represents a character.  So I'll check the
	// length of the string; if it's nonzero I'll send a character out
	// the pipe.
	if (ev->length > 0)
		serial.putChar(ev->string[0], false);
	return(1);
}

void CommPage::getCharIn(gpointer data,
						gint fd,
						GdkInputCondition condition) {
	
	char buf = serial.getChar();

	if (buf == '>')
		gui->insertText('\n');

	gui->insertText(buf);
}

void CommPage::setIO(bool connected) {
	if (connected) {
		gui->dbg(sprintf(gui->errmess, str.SETIOTRUDBG.c_str()));
		
		if (!serial.portIsOpen())
			serial.openPort();
		
		if (!serialTag)
			serialTag = gdk_input_add(serial.getFd(), GDK_INPUT_READ,
									  CommPage::getCharIn, NULL);
	} else {
		gui->dbg(str.SETIOFLSDBG.c_str());
		
		if (serialTag) {
			gdk_input_remove(serialTag);
			serialTag = 0;
			if (serial.portIsOpen())
				serial.closePort();
		}
	}
}

void CommPage::place_cursor()
{
	printf("got focus-in event\n");
	textArea.get_buffer()->place_cursor(textArea.get_buffer()->end());
	textArea.scroll_mark_onscreen(textArea.get_buffer()->get_insert());
}

Gui::Gui(int argc, char **argv, OptionTable& options, poptContext* pctx) :
  // >>>  Gtk::Main(PACKAGE, VERSION, argc, argv, options.getPoptionArray(), 0, pctx),
  Gtk::Main(argc, argv),
		 mainWin(options, tips)
{
	loader.checkPrereqs();
	mainWin.show_all();
	if (options[str.COMMFNTSOPT].getCStr() != NULL)
		setCommFont(options[str.COMMFNTSOPT].getCStr());
}

void Gui::setStatus(State newState) {
	state = newState;
	
	// we'll do this as a separate switch for everything we might need
	// to turn on or off

	// Which pages can we switch to?  We can always switch to the setup page; we can go to download
	// or comm unless we need to select a port.

	switch (state) {
	  case NEEDPORT:
		mainWin.controlPanel.get_tab_label(mainWin.controlPanel.commPage)->set_sensitive(false);
		mainWin.controlPanel.get_tab_label(mainWin.controlPanel.downloadPage)->set_sensitive(false);
		break;
		
	  default:
		mainWin.controlPanel.get_tab_label(mainWin.controlPanel.commPage)->set_sensitive(true);
		mainWin.controlPanel.get_tab_label(mainWin.controlPanel.downloadPage)->set_sensitive(true);
		break;
	}

	// Can we download?  We need a port and a file, and can't be
	// downloading.  In other words, we have to be IDLE
	switch (state) {
	  case IDLE:
		mainWin.controlPanel.get_tab_label(mainWin.controlPanel.downloadPage)->set_sensitive(true);
		mainWin.controlPanel.downloadPage.downloadButton.set_sensitive(true);
		break;

	  default:
		mainWin.controlPanel.downloadPage.downloadButton.set_sensitive(false);
	}

	// Can we cancel a download?  Only if we're currently downloading
	switch (state) {
	  case BOOTLOAD:
	  case SYNCWAIT:
	  case BULKERASE:
	  case PROMLOAD:
		mainWin.controlPanel.downloadPage.cancelButton.set_sensitive(true);
		break;

	  default:
		mainWin.controlPanel.downloadPage.cancelButton.set_sensitive(false);
		
	}

	// And put up the associated string
	switch (state) {
	  case NEEDPORT:
		mainWin.setStatus(str.NEEDPORTSTAT);
		break;

	  case NEEDFILE:
		mainWin.setStatus(str.NEEDFILESTAT);
		break;

	  case IDLE:
		mainWin.setStatus(str.IDLESTAT);
		break;

	  case BOOTLOAD:
		mainWin.setStatus(str.BOOTSTAT);
		break;

	  case SYNCWAIT:
		mainWin.setStatus(str.SYNCSTAT);
		break;

	  case BULKERASE:
		mainWin.setStatus(str.BULKSTAT);
		break;

	  case PROMLOAD:
		mainWin.setStatus(str.EEPROMSTAT);
		break;

	  case COMM:
		mainWin.setStatus(str.COMMSTAT);
		break;
	}
}

void Gui::dbg(const ustring debugstring) {
	if (debug) {
		fprintf(stderr, debugstring.c_str());
		fprintf(stderr, "\n");
		sync();
	}
}

void Gui::dbg(const int debugstringlength) {
	dbg(errmess);
}

void Gui::error(int len, int err)
{
	error(errmess, err);
}

void Gui::error(ustring errtext, int err)
{
	ustring buffer = errtext;
	
	if (err) {
		buffer += ": ";
		buffer += strerror(err);
	}

	// The commented-out code here is what I want to do, but
	// apparently the MessageBox ctor isn't complete; when it tries to
	// execute this, it prints a warning message and hangs.  So I've
	// got the C code below instead.
	
	//	vector<ustring> buttonList;
	//	buttonList.push_back(str.OKBUTERR.c_str());

	Gtk::MessageDialog message(buffer, false, Gtk::MESSAGE_ERROR);
	message.run();
	//	Gnome::MessageBox message(
	//							  buffer,
	//							  GNOME_MESSAGE_BOX_ERROR,
	//							  Gtk::SArray(buttonList));
	//	message.run();

	// >>>GtkWidget* message = gnome_message_box_new(buffer.c_str(),
	// >>>											GNOME_MESSAGE_BOX_ERROR,
	// >>>												str.OKBUTERR.c_str(),
	// >>>												NULL);
	// >>> gnome_dialog_run(GNOME_DIALOG(message));
	
}
