#ifndef _GUI_HH_
#define _GUI_HH_

#include "config.h"

#include <popt.h>

#include <vector>

#include <glibmm/ustring.h>
using Glib::ustring;

#include <gtkmm/adjustment.h>
#include <gtkmm/box.h>
#include <gtkmm/filechooserwidget.h>
#include <gtkmm/fontbutton.h>
#include <gtkmm/frame.h>
#include <gtkmm/label.h>
#include <gtkmm/main.h>
#include <gtkmm/menubar.h>
#include <gtkmm/notebook.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/scrollbar.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/separator.h>
#include <gtkmm/style.h>
#include <gtkmm/textview.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/window.h>

#include <pangomm/fontdescription.h>

#include "option.hh"

// at this point, I've gone very nearly from one extreme to the
// other.  The first time I wrote this, it was very close to having a
// class per container.  This time, it's got a class for the entire
// gui, a class for the application (I don't think these can be
// combined -- if they can, they should be), and a class for each page in
// the notebook.

// The philosophy used here is that every top-level window appearing
// on the screen should have a class, and every widget that obscures
// another widget should have a class (ie notebook pages).  We'll see
// how we like it when it's done...

//: SetupPage
// the page in the Setup notebook child that actually contains
// the setups.  I'm doing this as a separate class to avoid some
// annoying error messages -- if I do this stuff as part of the
// Preferences window, I get a changed() signal when I call setPath in
// the port window and there's an error message (though everything
// actually works).  This way the message is avoided.
class SetupPage : public Gtk::VBox {
	Gtk::HBox optionsOuterBox;
	Gtk::Frame optionsFrame;
	Gtk::VBox optionsLeftBox;
	Gtk::HBox optionsInnerBox[2];  // this can't be a buttonbox because
								   // buttonboxes make all buttons the
								   // same size.
	Gtk::VBox optionsRightBox;
	Gtk::FontButton commFontPicker;

	Gtk::Frame portFrame;
	Gtk::FileChooserWidget portBox;
	
	void newFont(gchar *fontName);
	void checkbox_toggled();
	void port_changed();
	void changed();
	
  public:
	SetupPage(OptionTable& options, Gtk::Tooltips& tips);
	void apply(OptionTable* options);

	//	 Pango::FontDescription getCommFont() {
	//		return Pango::FontDescription(commFontPicker.get_font_name());
	//	}

	ustring getCommFont() {
		return commFontPicker.get_font_name();
	}
	
	ustring getPortName() {
		return portBox.get_filename();
	}
};

//: Download
// Page used to download files
class DownloadPage : public Gtk::VBox {

	Gtk::Frame fileFrame;
	Gtk::HBox downloadButtonBox;

	// callbacks
	bool checks19file(const ustring& dirName, ustring& fileName);
	void fileSelected();
	void fileUnSelected();
	void downloadPressed();
	void cancelPressed();

  public:
	Gtk::FileChooserWidget fileBox;
	Gtk::Button downloadButton;
	Gtk::Button cancelButton;
	DownloadPage(OptionTable& options, Gtk::Tooltips& tips);
};

//: Page for serial interaction with hc11 board
class CommPage: public Gtk::VBox {

	//  Gtk::Frame windowFrame;
	Gtk::ScrolledWindow textBox;
	//	Gtk::Adjustment hadj;
	//	Gtk::Adjustment vadj;
	//	Gtk::VScrollbar textScroll;
	Glib::RefPtr<Gtk::TextBuffer::Tag> fontTag;
	Glib::RefPtr<Gtk::TextBuffer::TagTable> tagTable;
	
  public:
	Gtk::TextView textArea;
	Glib::RefPtr<Gtk::Style> textAreaStyle;
	
	bool open;
	gint serialTag;
	
	int sendCharOut(GdkEventKey *);
	static void getCharIn(gpointer data, gint fd, GdkInputCondition condition);
	
  public:
	bool winOpen() {return(open);}
	void setIO(bool connected);
	CommPage(OptionTable& options);

	void setCommFont(ustring newFont) {
		
		//		printf("setting font to %s\n", newFont.c_str());
		fontTag->property_font() = newFont;
		textArea.get_buffer()->apply_tag(fontTag, textArea.get_buffer()->begin(), textArea.get_buffer()->end());

		//		textArea.modify_font(Pango::FontDescription(newFont));
	}

	void place_cursor();
	
	void insertText(char inch) {
		if (inch != '\0') {
			ustring str(1, inch);

			textArea.get_buffer()->place_cursor(textArea.get_buffer()->end());
			textArea.get_buffer()->insert_at_cursor(str);
			textArea.scroll_mark_onscreen(textArea.get_buffer()->get_insert());
			//			textArea.get_buffer()->insert_at_cursor(str);
		}
	}
};

//: Control Panel for downloader
class ControlPanel : public Gtk::Notebook
{
	void pageChanged(GtkNotebookPage*  page, guint newPage);
	int currentPage;
	
  public:
	SetupPage setupPage;
	DownloadPage downloadPage;
	CommPage commPage;

	ControlPanel(OptionTable& options, Gtk::Tooltips& tips);

	void setCommFont(ustring newFont) {
		commPage.setCommFont(newFont);
	}
	
	void insertText(char inch) {
		commPage.insertText(inch);
	}

	bool winOpen() {
		return(commPage.winOpen());
	}

	void setIO(bool connected) {
		commPage.setIO(connected);
	}
	
};

//: Appbar
// I need to build myself an appbar, since I'm not using Gnome any more
class AppBar : public Gtk::HBox {
 private:
	Gtk::Label status;
	Gtk::ProgressBar progress;

 public:
	AppBar();
	void setStatus(Glib::ustring stattext);
	void setProgress(double percentage);
};

//: Main window for downloader
// class MainWin : public Gnome::App {
class MainWin : public Gtk::Window {
	static void DestroyHandler();
	static void ExitPressed();
	static void ShowManual();
	static void ShowAbout();
	
  public:
	Gtk::VBox vbox;
	Gtk::MenuBar menubar;
	ControlPanel controlPanel;
	//>>>	Gnome::AppBar appBar;
	AppBar appBar;

	void setStatus(ustring status);
	void setProgress(int sofar, int total);
	
	MainWin(OptionTable& options, Gtk::Tooltips& tips);

	void setCommFont(ustring newFont) {
		controlPanel.setCommFont(newFont);
	}
	
	void insertText(char inch) {
		controlPanel.insertText(inch);
	}

	bool winOpen() {
		return(controlPanel.winOpen());
	}

	void setIO(bool connected) {
		controlPanel.setIO(connected);
	}

};

//: Master interface to all GUI functions
class Gui : public Gtk::Main {

  public:
		
	typedef enum {
	  NEEDPORT, NEEDFILE, IDLE, BOOTLOAD, SYNCWAIT, BULKERASE, PROMLOAD, COMM
	} State;

  private:
	
	Gtk::Tooltips tips;
	State state;

  protected:

  public:
	MainWin mainWin;
	
	// for error messages
	static const int ERRBUFSIZE = 256;
	static char errmess[ERRBUFSIZE];
	
	Gui(int argc, char **argv, OptionTable& options, poptContext* pctx);
	
	void setStatus(State state);
	State getStatus() {return state;}

	void showProgress();
	
	void setProgress(int sofar, int total) {
		mainWin.setProgress(sofar, total);
	}

	void setCommFont(ustring newFont) {
		mainWin.setCommFont(newFont);
	}

	void insertText(char inch) {
		mainWin.insertText(inch);
	}

	bool winOpen() {
		return(mainWin.winOpen());
	}

	void setIO(bool connected) {
		mainWin.setIO(connected);
	}

	ustring getFileName() {
		return mainWin.controlPanel.downloadPage.fileBox.get_filename();
	}

	ustring getPortName() {
		return mainWin.controlPanel.setupPage.getPortName();
	}
	
	void dbg(const ustring debugustring);
	void dbg(const int debugustringlength);
	
	void error(const ustring errtext, int err=0);
	void error(int len, int err=0);

};
	
#endif
