// This is by no means a full wrapper; it just provides a mechanism
// for a class to include a popt command-line option table.  Also,
// I've integrated the gnome-config stuff in here, so my app can tread
// options from the command line and options from the configuration
// consistently.  Unfortunately, my Preferences class still has to
// know far too much about the options table, to set up its check
// boxes and things....

// As long as I'm writing this, and dumping the gnome config stuff,
// anyway, I'll integrate config file maintenance with option handling here
extern "C" {
    #include <assert.h>
    #include <stdio.h>
    #include <string.h>
}

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gui.hh"
#include "option.hh"
#include "globals.hh"
#include "strings.hh"

#define	strassign(str)	((str != 0) ? strcpy((char *)(malloc(strlen(str)+1)), str) : 0)

//void logoptions(void *thisptr, char *action, const void *fromptr)
//{
//	fprintf(stderr, "%s:  0x%08x", action, thisptr);
//	if (fromptr != NULL)
//		fprintf(stderr, " from 0x%08x", fromptr);
//	fprintf(stderr, "\n");
//}

Option::Option() :
  widgetm(NULL)
{
	//	logoptions(this, "empty constructor", NULL);
	
	data.longName = NULL;
	data.shortName = '\0';
	data.argInfo = 0;
	data.arg = 0;
	data.val = 0;
	data.descrip = NULL;
	data.argDescrip = NULL;
}

Option::Option(const Option& option)
{
	//	logoptions(this, "copy constructor", &option);
	
	widgetm = option.widgetm;
	//	if (option.data.longName != NULL)
	//		data.longName =	strdup(option.data.longName);
	//	else
	//		data.longName = NULL;
	data.longName = option.data.longName;
	
	data.shortName = option.data.shortName;
	data.argInfo = option.data.argInfo;
	data.arg = option.data.arg;
	data.val = option.data.val;
	
	//	if (option.data.descrip != NULL)
	//		data.descrip = strdup(option.data.descrip);
	//	else
	//		data.descrip = NULL;
	data.descrip = option.data.descrip;
	
	//if (option.data.argDescrip != NULL)
	//		data.argDescrip = strdup(option.data.argDescrip);
	//	else
	//		data.argDescrip = NULL;
	data.argDescrip = option.data.argDescrip;
}
	
Option::Option(const string& longName,
			   const string& shortName,
			   int argInfo,
			   void* arg,
			   int val,
			   const string& descrip,
			   const string& argDescrip) :
  widgetm(NULL)
{
	//	logoptions(this, "constructor", NULL);
	
	//	data.longName = strdup(longName.c_str());
	data.longName = longName.c_str();
	data.shortName = shortName[0];
	data.argInfo = argInfo;
	data.arg = arg;
	data.val = val;
	//	data.descrip = strdup(descrip.c_str());
	data.descrip = descrip.c_str();
	//	data.argDescrip = strdup(argDescrip.c_str());
	data.argDescrip = argDescrip.c_str();
}

Option::~Option()
{
	//	logoptions(this, "destructor", NULL);
	
	//	if (data.longName != NULL)
	//		free((void *) data.longName);

	//	if (data.descrip != NULL)
	//		free((void *) data.descrip);

	//	if (data.argDescrip != NULL)
	//		free((void *) data.argDescrip);
}

// accessors for metadata
const char * Option::longName() const
{
  return data.longName;
}

const char Option::shortName()
{
return data.shortName;
}

int Option::argInfo()
{
	return data.argInfo;
}

const char *Option::descrip()
{
	return data.descrip;
}

const char *Option::argDescrip()
{
	return data.argDescrip;
}

Gtk::Widget *Option::widget()
{
	return widgetm;
}

// accessors for data
bool Option::getBool()
{
	assert((data.argInfo == POPT_ARG_LONG) || (data.argInfo == POPT_ARG_NONE));
		
	return *(bool *) data.arg;
}

char *Option::getCStr()
{
	assert(data.argInfo == POPT_ARG_STRING);
	
	return *(char **) data.arg;
}

// mutators for metadata
void Option::setWidget(Gtk::Widget *newWidget)
{
	widgetm = newWidget;
}	

// mutators for data

// In this case, we'll assume the old string (if any) was malloced, and not shared with
// any other pointers.  We'll free it and strdup the new value (since it may well be a
// buffer we can't corrupt
void Option::set(char *value) {
	assert(data.argInfo == POPT_ARG_STRING);

	if (*(char **)(data.arg) != NULL)
		free(*(char **)(data.arg));
	*(char **)(data.arg) =  strdup(value);
}

void Option::set(bool value)
{
	assert((data.argInfo == POPT_ARG_LONG) || (data.argInfo == POPT_ARG_NONE));
	
	*(bool *)data.arg = value;
}

void Option::set(Glib::ustring value)
{
	set((char *)value.c_str());
}

void  bogus()
{}

OptionTable::OptionTable() {
	// The C routines assume a null entry at the end of the option
	// list.  I'm going to end up moving this back an element every
	// time I insert a new option, but that'll keep the data
	// structures internally consistent at all times
	Option *option;
	
	//>>>	option.longName = 0;
	//	option.shortName = 0;
	//	option.argInfo = 0;
	//	option.arg = 0;
	//	option.val = 0;
	//	option.descrip = 0;
	//	option.argDescrip = 0;

	bogus();

	//	logoptions(this, "optiontable constructor", NULL);
	option = new Option;
	push_back(*option);

	// now let's open the config files.
	ustring etcname = ustring(SYSCONFDIR) + ustring("/") + PACKAGE;
	configfilename = ustring(getenv("HOME")) + "/." + PACKAGE;
	  
	etcfile = fopen(etcname.c_str(), "r+");
	dotfile = fopen(configfilename.c_str(), "r+");
	if (dotfile == NULL)
		dotfile = fopen(configfilename.c_str(), "w+");
}

//OptionTable::~OptionTable() {

	// I need to explicitly free the strings I put in the table.
//	for (iterator i = begin(); i != end(); ++i) {
//		if (i->longName() != 0) {
//			free((char *)(i->longName));
//		}
//		if (i->descrip != 0) {
//			free((char *)(i->descrip));
//		}
//		if (i->argDescrip != 0)
//			free((char *)(i->argDescrip));
//	}
//}

void OptionTable::put(const Option& option)
{
	//	fprintf(stderr, "\ninserting 0x%08x: '%s'\n", &option, option.longName());
	//	insert(begin(), option);
	//	fprintf(stderr, "contents of table before insert:\n");
	//	for (iterator i = begin(); i != end(); i++)
	//		fprintf(stderr, "    0x%08x:  '%s'\n", &(*i), i->longName());
		
	insert(end()-1, option);

	//	fprintf(stderr, "contents of table after insert:\n");
	//	for (iterator i = begin(); i != end(); i++)
	//		fprintf(stderr, "    0x%08x: '%s'\n", &(*i), i->longName());
	
}

// The right way to do this is to use POPT_ARG_INCLUDE_TABLE and have
// nested tables.  But if I hook up the vectors like that, the
// Gnome routines won't work with it, and if I hook up the arrays
// contained in the vectors, I can't use STL vector operations at any
// level but the top when I build the configuration window.  Instead,
// I'll make one long option list out of them.
//void OptionTable::merge(OptionTable& newoptions) {

	// I need to insert all but the last (ie the null) element of the
	// newoptions just before the last-but-1 (once again, the null)
	// element of my options.
//	insert(end()-1, newoptions.begin(), newoptions.end()-1);
	
	// Oh, boy.  poptOptions are a mess from my POV, because there are
	// various void pointers that I would need to interpret to do
	// things right.  So, I'll assume that once a subtable has been
	// merged into the main table, it will never be used again (DANGER
	// WILL ROBINSON!) and I'll delete its contents.
//	newoptions.erase(newoptions.begin(), newoptions.end()-1);
//}

// evil stuff -- but that's how we get back to interact with the C
// routines
// >>> struct poptOption* OptionTable::getPoptionArray() {
//	return &front();
//}

// >>> this is going to be a pretty complete rewrite -- I think I want
// to go ahead and read both the global and local config files from
// beginning to end, and as I get each option from the file search
// for it in the option table and put it in.  This won't really be so
// bad, as I can increment through the option table as long as
// things are in order.
void OptionTable::read(void)
{
	if (etcfile != NULL)
		readConfigurationFromFile(etcfile);
	readConfigurationFromFile(dotfile);

/*
	for (OptionTable::iterator i = begin(); i != end(); ++i)
		if (i->longName != NULL) {
			std::string path;
			path = "/gdl/Options/" + (std::string)(i->longName);
			
			switch (i->argInfo) {
			  case POPT_ARG_LONG:
				//	>>>			*(int *)i->arg = gnome_config_get_int(path.c_str());
				break;
				
			  case POPT_ARG_STRING:
				//	>>>			*(char **)i->arg = gnome_config_get_string(path.c_str());
				break;
				
			  default:
				break;
			}
		}
*/
}

// read all the configuration items that are in the file.  I hope  the
// value fields are long enough....
void OptionTable::readConfigurationFromFile(FILE *file)
{
	char *key, *value;
	char inbuf[BUFSIZE];
	int longVal;
	rewind(file);
	do {
		key = fgets(inbuf, BUFSIZ, file);
		if (key != NULL) {
			value = strtok(inbuf, "=");
			value = strtok(NULL, "=");
			if ((value != NULL) &&
				(value[strlen(value)-1] == '\n'))
				value[strlen(value)-1] = 0;
			for (iterator i = begin(); i != end(); ++i) {
				if ((i->longName() != NULL) && strcmp(i->longName(), key) == 0) {
					switch (i->argInfo()) {
					  case POPT_ARG_LONG:
						sscanf(value, "%d", &longVal);
						i->set(longVal);
						continue;

					  case POPT_ARG_STRING:
						i->set(value);
						continue;
						break;

					  default:
						break;
					}
				}
			}
		}
	} while (key != NULL);
}

// write all the configuration items to the dot file.
void OptionTable::write(void) {

	dotfile = fopen(configfilename.c_str(), "w+");
	if (dotfile != NULL)
		for (OptionTable::iterator i = begin(); i != end(); ++i)
			if (i->longName() != NULL) {
				switch (i->argInfo()) {
				  case POPT_ARG_LONG:
					fprintf(dotfile, "%s=%d\n", i->longName(), i->getBool());
					break;
			
				  case POPT_ARG_STRING:
					fprintf(dotfile, "%s=%s\n", i->longName(), i->getCStr());
					break;
					
				  default:
					break;
				}
			}
	fflush(dotfile);
}

Option& OptionTable::search(const char shortName)
{
	for (iterator i = begin(); i != end(); ++i)
		if (i->shortName() == shortName)
			return  *i;
	
	gui->error(sprintf(gui->errmess, str.NOGBOOLERR.c_str(), shortName, shortName));
}

Option& OptionTable::operator[](const char shortName)
{
	return search(shortName);
}

Option& OptionTable::operator[](const ustring& shortName)
{
	return search(shortName[0]);
}
