//
//  $Log: serial.cc,v $
//  Revision 1.2  2000/08/03 03:58:31  pfeiffer
//  More changes than I even want to think about, upgrading to match
//  current gtk+ and gtkmm
//
//
#include "config.h"
extern "C" {
#include <linux/serial.h>
  
#include <sys/types.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/stat.h>

    #if HAVE_DIRENT_H
    # include <dirent.h>
    # define NAMLEN(dirent) strlen((dirent)->d_name)
    #else
    # define dirent direct
    # define NAMLEN(dirent) (dirent)->d_namlen
    # if HAVE_SYS_NDIR_H
    #  include <sys/ndir.h>
    # endif
    # if HAVE_SYS_DIR_H
    #  include <sys/dir.h>
    # endif
    # if HAVE_NDIR_H
    #  include <ndir.h>
    # endif
    #endif

#include <errno.h>
#include <fcntl.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#define	__need_timeval
#include <time.h>
#include <unistd.h>
};

#include <sstream>
#include <string>
#include <algorithm>
#include "gui.hh"
#include "serial.hh"
#include "globals.hh"

Serial::Serial() {
  fd = -1;
  // >>>  options.putPoptionInTable(str.PORTLOPT, str.PORTSOPT, POPT_ARG_STRING, &portName, (int) gnome_config_get_string("/gdl/Loader/serialPort"), str.PORTTIP, string(""));
  // >>>  options.put(*(new Option(str.PORTLOPT, str.PORTSOPT, POPT_ARG_STRING, &portName, (int) "/dev/ttyS0", str.PORTTIP, string(""))));
  options.put(*(new Option(str.PORTLOPT, str.PORTSOPT, POPT_ARG_STRING, &portName, 0, str.PORTTIP, string(""))));
}

// OK, time to give up on all the ugly hacks I had in here to check
// for serial ports.  Since I'm only testing under Linux, I'll look
// for file names of the form ttyS*, and symbolic links that point to
// files with names of that form.  I'll filter this with
// ``writeable.''  If somebody ports this to another OS, they can deal
// with what ttys are named on that system (noting that under Solaris,
// they seem to be named ttya and ttyb).
bool Serial::checkDev(const string& dirName, string& fileName) {
	const string devName = dirName + fileName;
	const char *cname = devName.c_str();

	gui->dbg(sprintf(gui->errmess, str.TRYDEVDBG.c_str(), devName.c_str()));

	// Is its name of the form ttyS*?
	if (strstr(cname, "ttyS") == NULL) {
		gui->dbg(str.BADNMDBG.c_str());
		
		// is it a symbolic link pointing to something of the form ttyS*?
		int slinklen;
		char slinkname[FILENAME_MAX+1];
		slinklen = readlink(cname, slinkname, FILENAME_MAX);
		if (slinklen < 0)
			return false;
		
		slinkname[slinklen] = '\0';
		gui->dbg(sprintf(gui->errmess, str.SLNKNMDBG.c_str(), slinkname));
		if (strstr(slinkname, "ttyS") != slinkname)
			return false;
		// I'd like to display the translation of the link when a file
		// name is a symbolic link, but this has some implications for
		// opening the file later that I don't want to explore right now...
		//		else
		//			fileName.append(" -> " + string(slinkname));
		gui->dbg(sprintf(gui->errmess, str.LINKDBG.c_str(), devName.c_str()));
	}
	
	// Now -- can I read and write the file?
	
	gui->dbg(sprintf(gui->errmess, str.OPENDBG.c_str(), cname));
	int fd = open(cname, O_RDWR | O_NOCTTY | O_NDELAY);
	gui->dbg(sprintf(gui->errmess, str.PORTDBG.c_str(), fd));
	if (fd < 0) return false;
	close(fd);
	
	return true;
}

bool Serial::portIsOpen() {
	return (fd > 0);
}

bool Serial::openPort() {
	
	gui->dbg(sprintf(gui->errmess, str.NAMEDBG.c_str(), gui->getPortName().c_str()));
	
  	fd = open(gui->getPortName().c_str(), O_RDWR | O_NDELAY | O_NOCTTY);
	
	gui->dbg(sprintf(gui->errmess, str.PORTDBG.c_str(), fd));
	
	if (fd < 0) {
		gui->error(str.PORTNOACCERR, errno);
		return(false);
	}

	fcntl(fd, F_SETFL, 0);
	
	return(true);
}

bool Serial::closePort() {
	if (fd <= 0) {
		gui->error(str.NOTOPENERR);
		return(false);
	}

	if (close(fd)) {
		gui->error(str.PORTCLOSEERR, errno);
		return(false);
	}

	fd = -1;
	return(true);
}

void Serial::dumpAttrs(struct termios termios) {
	
	fprintf(stderr, str.ATTRSDBG.c_str());
	fprintf(stderr, "    c_iflag  0x%08x\n", termios.c_iflag);
	fprintf(stderr, "    c_oflag  0x%08x\n", termios.c_oflag);		
	fprintf(stderr, "    c_cflag  0x%08x\n", termios.c_cflag);		
	fprintf(stderr, "    c_lflag  0x%08x\n", termios.c_lflag);		
	fprintf(stderr, "    c_line   0x%08x\n", termios.c_line);
	fprintf(stderr, "    c_ispeed 0x%08x\n", termios.c_ispeed);
	fprintf(stderr, "    c_ospeed 0x%08x\n", termios.c_ospeed);

	// looks like TIOCGSERIAL has gone away...
	/*
	struct serial_struct serinfo;
	serinfo.reserved_char[0] = 0;
	if (ioctl(fd, TIOCGSERIAL, &serinfo) < 0)
		perror(str.NOSERDBGERR.c_str());
	else {
		
		fprintf(stderr, "type 0x%08x\n", serinfo.type);
		fprintf(stderr, "line 0x%08x\n", serinfo.line);
		fprintf(stderr, "port 0x%08x\n", serinfo.port);
		fprintf(stderr, "irq 0x%08x\n", serinfo.irq);
		fprintf(stderr, "flags 0x%08x\n", serinfo.flags);
		fprintf(stderr, "xmit_fifo_size 0x%08x\n", serinfo.xmit_fifo_size);
		fprintf(stderr, "custom_divisor 0x%08x\n", serinfo.custom_divisor);
		fprintf(stderr, "baud_base 0x%08x\n", serinfo.baud_base);
		fprintf(stderr, "close_delay 0x%08x\n", serinfo.close_delay);
#if IO_TYPE_DEFINED
		fprintf(stderr, "io_type 0x%08x\n", serinfo.io_type);
#endif
		fprintf(stderr, "reserved_char[0] 0x%08x\n", serinfo.reserved_char[0]);
		fprintf(stderr, "hub6 0x%08x\n", serinfo.hub6);
		fprintf(stderr, "closing_wait 0x%08x\n", serinfo.closing_wait);
		fprintf(stderr, "closing_wait2 0x%08x\n", serinfo.closing_wait2);
#if IOMEM_BASE_DEFINED
		fprintf(stderr, "iomem_base 0x%08x\n", serinfo.iomem_base);
#endif
#if IOMEM_REG_SHIFT_DEFINED
		fprintf(stderr, "iomem_reg_shift 0x%08x\n", serinfo.iomem_reg_shift);
#endif
#if PORT_HIGH_DEFINED
		fprintf(stderr, "port_high 0x%08x\n", serinfo.port_high);
#endif
		fprintf(stderr, "reserved[0] 0x%08x\n", serinfo.reserved[0]);
	}
	*/
}
	
bool Serial::initX(speed_t speed) {
	struct termios termios;
	struct timespec req;

	if (tcgetattr(fd, &termios)) {
		gui->error(str.GETATTRERR, errno);
		return(false);
	}
	
	if (debug)
		dumpAttrs(termios);

	// code stolen from example in serial-programming-HOWTO
	 bzero(&termios, sizeof(termios));
     termios.c_cflag = CS8 | CLOCAL | CREAD;
     termios.c_iflag = IGNBRK;
     termios.c_lflag = 0;
     termios.c_oflag = 0;
     
	termios.c_cc[VMIN] = 0;
	termios.c_cc[VTIME]= 3;
	
	// I don't have any docs on how to set up c_line
	termios.c_line = 0;
	
	if (cfsetispeed(&termios, speed)) {
		gui->error(str.ISPDERR, errno);
		return(false);
	}
	
	if (cfsetospeed(&termios, speed)) {
		gui->error(str.OSPDERR, errno);
		return(false);
	}
	
	tcdrain(fd);

	// Now wait a bit before actually making the change, for the other
	// end.
	req.tv_sec = 0;
	req.tv_nsec = 100000000;

	nanosleep(&req, NULL);

	gui->dbg(sprintf(gui->errmess, str.PATTRSDBG.c_str(), fd));
	
	if (tcsetattr(fd, TCSADRAIN, &termios)) {
		gui->error(str.SETATTRERR, errno);
		return(false);
	}
	
	if (debug) {
		
		if (tcgetattr(fd, &termios)) {
			gui->error(str.GETATTRERR, errno);
			return(false);
		}
		
		dumpAttrs(termios);
	}
	
	return(true);
}

bool Serial::init1200() {
	return initX(B1200);
}

bool Serial::init9600() {
	return initX(B9600);
}

// I'll use -2 for an error return; -1 for ``no response'' and a positive value is
// a character actually returned
int Serial::getChar() {
    unsigned char c;
    int status;
	
    status = read(fd, &c, 1);
    if (status < 0) {
		gui->error(str.CNTRDERR, errno);
		return(-2);
    }
		
	if (status == 0) {
		gui->dbg(str.GETNOCHARDBG.c_str());
		return(-1);
	}

	gui->dbg(sprintf(gui->errmess, str.GTCHRDBG.c_str(), c));
	
	return (int) c & 0xff;
}

bool Serial::putChar(unsigned int c, bool echo) {
    unsigned char x = c;
	int i;
	
    if (write(fd, &x, 1) != 1) {
		gui->error(str.CNTWRTERR, errno);
		return(false);
	}

	tcdrain(fd);
	gui->dbg(sprintf(gui->errmess, str.PUTCHARDBG.c_str(), x));
	
	if (!echo)
		return(true);

	i = getChar();
  
	if (c != i) { // make sure it's echoed right
		gui->error(sprintf(gui->errmess, str.ECHOERR.c_str(), c, i));
		return(false);
	}
  
	return(true);
}
	
int Serial::getLastChar() {
    int ch, old;

    old = ch = getChar();
    while (ch > -1) {
		old = ch;
		ch = getChar();
    }
	if (ch >= 0)
		return(old);
	else
		return(ch);
}

void Serial::flush() {
	int c;
	
    while ((c = getChar()) > -1);
}

bool Serial::drain() {
	if (!tcdrain(fd))
		return(true);
	else
		return(false);
}

