extern "C" {
	#include <libintl.h>
};

#include "strings.hh"

#define _(string) gettext(string)

Str::Str() :
    // program options strings
	BLKERASELOPT(_("bulk-erase")),
	BLKERASESOPT(_("e")),
	BLKERASETIP(_("bulk Erase")),
	
	BTSTRPLOPT(_("bootstrap-feedback")),
	BTSTRPSOPT(_("b")),
	BTSTRPTIP(_("Bootstrap feedback")),
	
	HDWRLOPT(_("hardware-echo")),
	HDWRSOPT(_("h")),
	HDWRTIP(_("Hardware echo")),

	STRTLOPT(_("start-addr")),
	STRTSOPT(_("s")),
	STRTTIP(_("Program start address specified in file")),
	
	MNEELOPT(_("min-eeprom")),
	MNEESOPT(_("m")),
	MNEETIP(_("Min eeprom address")),
	
	FILELOPT(_("file")),
	FILESOPT(_("f")),
	FILETIP(_("File to download")),
	
	DEBUGLOPT(_("debug")),
	DEBUGSOPT(_("d")),
	DEBUGTIP(_("enable debugging output")),
	
	PORTLOPT(_("port")),
	PORTSOPT(_("p")),
	PORTTIP(_("serial port")),

	COMMFNTLOPT(_("commfont")),
	COMMFNTSOPT(_("c")),
	COMMFNTTIP(_("Font Used in Communication Window")),

	LABELFNTLOPT(_("labelfont")),
	LABELFNTSOPT(_("l")),
	LABELFNTTIP(_("Font Used in Labels and Menus")),

	// tips
	DOWNLOADTIP(_("Download")),
	COMMTIP(_("Communicate with hc11")),
	DWNLDTIP(_("Download program to hc11")),
	QUITTIP(_("Quit gdl")),

	// errors
	NOERR(_(" No error ")),
	OKBUTERR(_("OK")),
	ADDRERR(_("User Program Error:  Attempted to write data to 0x%04x, lower than EEPROM min 0x%04x ")),
	
	CANTOPENERR(_("File Selection Error:  Unable to open ")),
	EMPTYERR(_("File Selection Error:  Empty program file ")),
	
	BADHEXERR(_("File format Error:  Illegal hex digit 0x%02x ")),
	FMTERR(_("File format Error:  Program file not an S19 file ")),
	S23ERR(_("Unhandled Format:  GDL doesn't handle 24 or 32 bit address (S2 and S3) ")),
	
	NOSERPORTS(_("Serial Port Error:  No useable serial ports")),
	PORTNOACCERR(_("Serial Port Error:  Port not accessible")),
	
	CNTRDERR(_(" Communications Error:  Unable to read character from board ")),
	CNTWRTERR(_(" Communications Error:  Unable to write character to port ")),
	ECHOERR(_(" Communications Error:  Sent 0x%02x)), hardware echoed 0x%02x ")),
	SYNCERR(_(" Communications Error:  Failed to sync with miniboard ")),
	BULKSYNCERR(_(" Communications Error:  Synchronization failure after bulk erase ")),
	RESPERR(_(" Communications Error:  No response from board ")),
	
	MEMERR(_(" Board Memory Error:  At address 0x%04x expected 0x%02x got 0x%02x ")),
	
	PORTNOSPECERR(_("Internal Error:  No serial port specified ")),
	BUFOVFLOWERR(_("Internal Error:  buffer overflow converting format string <%s> (_(contact pfeiffer@cs.nmsu.edu))")),
	BUFTOOSMALLERR(_("Internal Error:  buffer not long enough for lines in program file (_(contact pfeiffer@cs.nmsu.edu))")),
	CLOSENULLERR(_("Internal Error:  Tried to close null program stream (_(contact pfeiffer@cs.nmsu.edu))")),
	PROGCLOSEERR(_("Internal Error:  Closing program stream (_(contact pfeiffer@cs.nmsu.edu))")),
	NOTOPENERR(_("Internal Error:  Tried to close non-open port (_(contact pfeiffer@cs.nmsu.edu))")),
	PORTCLOSEERR(_("Internal Error:  Error closing port (_(contact pfeiffer@cs.nmsu.edu))")),
	GETATTRERR(_("Internal Error:  Unable to get port attributes (_(contact pfeiffer@cs.nmsu.edu))")),
	ISPDERR(_("Internal Error:  Unable to set port input speed (_(contact pfeiffer@cs.nmsu.edu))")),
	OSPDERR(_("Internal Error:  Unable to set port output speed (_(contact pfeiffer@cs.nmsu.edu))")),
	SETATTRERR(_("Internal Error:  Unable to set port attributes (_(contact pfeiffer@cs.nmsu.edu))")),
	SELERR(_("Internal Error:  Select failure (_(contact pfeiffer@cs.nmsu.edu))")),
	STATERR(_("Internal Error:  Unable to stat file %s (_(contact pfeiffer@cs.nmsu.edu")),
	NOGBOOLERR(_("Internal Error:  Tried to get bool value of nonexistent option 0x%02x(_(%c)) (_(contact pfeiffer@cs.nmsu.edu")),
	NOGSTRNGERR(_("Internal Error:  Tried to get string value of nonexistent option 0x%02x(_(%c)) (_(contact pfeiffer@cs.nmsu.edu")),
	NOSBOOLERR(_("Internal Error:  Tried to set bool value of nonexistent option 0x%02x(_(%c)) (_(contact pfeiffer@cs.nmsu.edu")),
	NOSSTRNGERR(_("Internal Error:  Tried to set string value of nonexistent option 0x%02x(_(%c)) (_(contact pfeiffer@cs.nmsu.edu")),
	NOSERDBGERR(_("cannot get serial info\n")),

	// debug
	SNDCHRDBG(_("Keystroke 0x%02x\n")),
	GTCHRDBG(_("Getchar 0x%02x\n")),
	ATTRSDBG(_("Communication attributes:\n")),
	PATTRSDBG(_("about to set attributes for fd %d\n")),
	GETNOCHARDBG(_("No character returned by getchar(_())\n")),
	PUTCHARDBG(_("putChar 0x%02x\n")),
	OPENDIR1DBG(_("Entering openDir %s\n")),
	OPENDIR2DBG(_("Exiting openDir %s\n")),
	XPND1DBG(_("Entering expandDir %s\n")),
	XPND2DBG(_("Exiting expandDir %s\n")),
	CLSDIR1DBG(_("Entering closeDir %s\n")),
	CLSDIR2DBG(_("Exiting closeDir %s\n")),
	CLPSDIR1DBG(_("Entering collapseDir %s\n")),
	CLPSDIR2DBG(_("Exiting collapseDir %s\n")),
	CHCKFILDBG(_("Checking file %s\n")),
	BADNMDBG(_("Wrong form of file name\n")),
	SLNKNMDBG(_("Links to %s\n")),
	DNLDPRESDBG(_("Download clicked, port is %s, file is %s\n")),
	FLSTATDBG(_("About to stat file\n")),
	STATDBG(_("Setting status to %s\n")),
	SETIOTRUDBG(_("setIO true\n")),
	SETIOFLSDBG(_("setIO false\n")),
	TRYDEVDBG(_("Trying device file %s\n")),
	OPENDBG(_("Opening file %s\n")),
	IOCTLDBG(_("IOCTL exists\n")),
	NAMEDBG(_("Port name is %s\n")),
	PORTDBG(_("Port fd is %d\n")),
	LINKDBG(_("Found good symbolic link %s\n")),
	RCDDBG(_("Download record at 0x%04x")),

	// labels
	LABELSLBL(_("Labels")),
	COMMLBL(_("Communication")),
	CNSTFONTLBL(_("Font for Controls and Labels")),
	DWNLDLBL(_("Download")),
	CNCLBL(_("Cancel")),
	DOWNTABLBL(_("Download")),
	COMMTABLBL(_("Communicate")),
	SETUPTABLBL(_("Setup")),

	// menus
	EXITMENU(_("_Exit")),
	FILEMENU(_("_File")),
	HELPMENU(_("_Help")),
	MANUALMENU(_("_Gdl Manual")),
	ABOUTMENU(_("_About")),

	// program status
	IDLESTAT(_("Ready to download")),
	BOOTSTAT(_("Downloading bootstrap loader")),
	SYNCSTAT(_("Syncing with HC11")),
	BULKSTAT(_("Bulk erasing EEPROM")),
	EEPROMSTAT(_("Downloading program to EEPROM")),
	NEEDPORTSTAT(_("Need to set serial port")),
	NEEDFILESTAT(_("Need to select file to download")),
	COMMSTAT(_("Using serial link to communicate with HC11")),
	
	// about
	MYNAME(_("Joseph J. Pfeiffer, Jr. (_(pfeiffer@cs.nmsu.edu))")),
	CAVEAT(_("Portions of loader.cc derived from Fred Martin's dlm20")),
	LICENSE(_("COPYRIGHT (C)) 2001 Licensed under the GNU Public License"))
{}

